@@ -11,6 +11,7 @@ import numpy as np
 import copy
 import collections
 
+
 class Server(BasicServer):
     def initialize(self, *args, **kwargs):
         self.init_algo_para({'learning_rate_lambda': 0.01})
@@ -47,12 +48,12 @@ class Server(BasicServer):
             res.append(max(p[i] + lmbd, 0))
         return res
 
-    def global_test(self, flag='valid'):
+    def global_test(self, flag='val'):
         """
-        Validate accuracies and losses on clients' local datasets
-        :param
-            dataflag: choose train data or valid data to evaluate
-        :return
+        Validate accuracies and losses on clients' local_movielens_recommendation datasets
+        Args:
+            flag: choose train data or valid data to evaluate
+        Returns:
             metrics: a dict contains the lists of each metric_value of the clients
         """
         all_metrics = collections.defaultdict(list)
@@ -64,10 +65,12 @@ class Server(BasicServer):
 
     def test(self, model=None, flag='test'):
         if model == None: model = self.result_model
-        data = self.test_data if flag=='test' else self.valid_data
-        if data is None: return {}
+        data = self.test_data if flag == 'test' else self.val_data
+        if data is None:
+            return {}
         else:
-            return self.calculator.test(model, self.test_data, batch_size = self.option['test_batch_size'])
+            return self.calculator.test(model, self.test_data, batch_size=self.option['test_batch_size'])
+
 
 class Client(BasicClient):
     def reply(self, svr_pkg):
@@ -81,4 +84,4 @@ class Client(BasicClient):
         return {
             "model": model,
             "loss": loss,
-        }
\ No newline at end of file
+        }
