@@ -1,3 +1,7 @@
+"""
+This is a non-official implementation of 'Federated Optimization in Heterogeneous
+Networks' (http://arxiv.org/abs/1812.06127)
+"""
 from flgo.algorithm.fedbase import BasicServer, BasicClient
 import copy
 import torch
