@@ -4,7 +4,7 @@ All the Partitioner should implement the method `__call__(self, data)`
 where `data` is the dataset to be partitioned and the return is a list of the partitioned result.
 
 For example, The IIDPartitioner.__call__ receives a indexable object (i.e. instance of torchvision.datasets.mnsit.MNSIT)
-and I.I.D. selects samples' indices in the original dataset as each client's local data.
+and I.I.D. selects samples' indices in the original dataset as each client's local_movielens_recommendation data.
 The list of list of sample indices are finally returnerd (e.g. [[0,1,2,...,1008], ...,[25,23,98,...,997]]).
 
 To use the partitioner, you can specify Partitioner in the configuration dict for `flgo.gen_task`.
@@ -19,8 +19,11 @@ from abc import abstractmethod, ABCMeta
 import random
 import numpy as np
 import collections
-
 import torch
+try:
+    import community.community_louvain
+except:
+    pass
 
 class AbstractPartitioner(metaclass=ABCMeta):
     @abstractmethod
@@ -36,11 +39,21 @@ class BasicPartitioner(AbstractPartitioner):
         return
 
     def register_generator(self, generator):
-        """Register the generator as an attribute that is accessible"""
+        r"""Register the generator as an self's attribute"""
         self.generator = generator
 
     def data_imbalance_generator(self, num_clients, datasize, imbalance=0):
-        """Split a data size into several parts"""
+        r"""
+        Split the data size into several parts
+
+        Args:
+            num_clients (int): the number of clients
+            datasize (int): the total data size
+            imbalance (float): the degree of data imbalance across clients
+
+        Returns:
+            a list of integer numbers that represents local_movielens_recommendation data sizes
+        """
         if imbalance == 0:
             samples_per_client = [int(datasize / num_clients) for _ in range(num_clients)]
             for _ in range(datasize % num_clients): samples_per_client[_] += 1
@@ -93,7 +106,7 @@ class IIDPartitioner(BasicPartitioner):
 
     Args:
         num_clients (int, optional): the number of clients
-        imbalance (float, optional): the degree of imbalance of the amounts of different local data (0<=imbalance<=1)
+        imbalance (float, optional): the degree of imbalance of the amounts of different local_movielens_recommendation data (0<=imbalance<=1)
     """
     def __init__(self, num_clients=100, imbalance=0):
         self.num_clients = num_clients
@@ -118,15 +131,15 @@ class DirichletPartitioner(BasicPartitioner):
     Args:
         num_clients (int, optional): the number of clients
         alpha (float, optional): `alpha`(i.e. alpha>=0) in Dir(alpha*p) where p is the global distribution. The smaller alpha is, the higher heterogeneity the data is.
-        imbalance (float, optional): the degree of imbalance of the amounts of different local data (0<=imbalance<=1)
+        imbalance (float, optional): the degree of imbalance of the amounts of different local_movielens_recommendation data (0<=imbalance<=1)
         error_bar (float, optional): the allowed error when the generated distribution mismatches the distirbution that is actually wanted, since there may be no solution for particular imbalance and alpha.
-        flag_index (int, optional): the index of the distribution-dependent (i.e. label) attribute in each sample.
+        index_func (func, optional): to index the distribution-dependent (i.e. label) attribute in each sample.
     """
-    def __init__(self, num_clients=100, alpha=1.0, error_bar=1e-6, imbalance=0, flag_index=-1):
+    def __init__(self, num_clients=100, alpha=1.0, error_bar=1e-6, imbalance=0, index_func=lambda X:[xi[-1] for xi in X]):
         self.num_clients = num_clients
         self.alpha = alpha
         self.imbalance = imbalance
-        self.flag_index = flag_index
+        self.index_func = index_func
         self.error_bar = error_bar
 
     def __str__(self):
@@ -135,15 +148,16 @@ class DirichletPartitioner(BasicPartitioner):
         return name
 
     def __call__(self, data):
-        attrs = [d[self.flag_index] for d in data]
+        attrs = self.index_func(data)
         num_attrs = len(set(attrs))
         samples_per_client = self.data_imbalance_generator(self.num_clients, len(data), self.imbalance)
         # count the label distribution
         lb_counter = collections.Counter(attrs)
+        lb_names = list(lb_counter.keys())
         p = np.array([1.0 * v / len(data) for v in lb_counter.values()])
         lb_dict = {}
         attrs = np.array(attrs)
-        for lb in range(len(lb_counter.keys())):
+        for lb in lb_names:
             lb_dict[lb] = np.where(attrs == lb)[0]
         proportions = [np.random.dirichlet(self.alpha * p) for _ in range(self.num_clients)]
         while np.any(np.isnan(proportions)):
@@ -189,9 +203,9 @@ class DirichletPartitioner(BasicPartitioner):
             loop_count += 1
         local_datas = [[] for _ in range(self.num_clients)]
         self.dirichlet_dist = []  # for efficiently visualizing
-        for lb in lb_counter.keys():
+        for lb in lb_names:
             lb_idxs = lb_dict[lb]
-            lb_proportion = np.array([pi[lb] * si for pi, si in zip(proportions, samples_per_client)])
+            lb_proportion = np.array([pi[lb_names.index(lb)] * si for pi, si in zip(proportions, samples_per_client)])
             lb_proportion = lb_proportion / lb_proportion.sum()
             lb_proportion = (np.cumsum(lb_proportion) * len(lb_idxs)).astype(int)[:-1]
             lb_datas = np.split(lb_idxs, lb_proportion)
@@ -209,20 +223,20 @@ class DiversityPartitioner(BasicPartitioner):
     Args:
         num_clients (int, optional): the number of clients
         diversity (float, optional): the ratio of locally owned types of the attributes (i.e. the actual number=diversity * total_num_of_types)
-        imbalance (float, optional): the degree of imbalance of the amounts of different local data (0<=imbalance<=1)
-        flag_index (int, optional): the index of the distribution-dependent (i.e. label) attribute in each sample.
+        imbalance (float, optional): the degree of imbalance of the amounts of different local_movielens_recommendation data (0<=imbalance<=1)
+        index_func (int, optional): the index of the distribution-dependent (i.e. label) attribute in each sample.
     """
-    def __init__(self, num_clients=100, diversity=1.0, flag_index=-1):
+    def __init__(self, num_clients=100, diversity=1.0, index_func=lambda X:[xi[-1] for xi in X]):
         self.num_clients = num_clients
         self.diversity = diversity
-        self.flag_index = flag_index
+        self.index_func = index_func
 
     def __str__(self):
         name = "div{:.1f}".format(self.diversity)
         return name
 
     def __call__(self, data):
-        labels = [d[self.flag_index] for d in data]
+        labels = self.index_func(data)
         num_classes = len(set(labels))
         dpairs = [[did, lb] for did, lb in zip(list(range(len(data))), labels)]
         num = max(int(self.diversity * num_classes), 1)
@@ -266,17 +280,17 @@ class GaussianPerturbationPartitioner(BasicPartitioner):
 
     Args:
         num_clients (int, optional): the number of clients
-        imbalance (float, optional): the degree of imbalance of the amounts of different local data (0<=imbalance<=1)
+        imbalance (float, optional): the degree of imbalance of the amounts of different local_movielens_recommendation data (0<=imbalance<=1)
         sigma (float, optional): the degree of feature skew
         scale (float, optional): the standard deviation of noise
-        feature_index (int, optional): the index of the feature to be processed for each sample.
+        index_func (int, optional): the index of the feature to be processed for each sample.
     """
-    def __init__(self, num_clients=100, imbalance=0.0, sigma=0.1, scale=0.1, feature_index=0):
+    def __init__(self, num_clients=100, imbalance=0.0, sigma=0.1, scale=0.1, index_func=lambda X:[xi[0] for xi in X]):
         self.num_clients = num_clients
         self.imbalance = imbalance
         self.sigma = sigma
         self.scale = scale
-        self.feature_index = feature_index
+        self.index_func = index_func
 
     def __str__(self):
         name = "perturb_gs{:.1f}_{:.1f}".format(self.sigma, self.scale)
@@ -284,7 +298,7 @@ class GaussianPerturbationPartitioner(BasicPartitioner):
         return name
 
     def __call__(self, data):
-        shape = tuple(np.array(data[0][self.feature_index].shape))
+        shape = tuple(np.array(self.index_func(data)[0].shape))
         samples_per_client = self.data_imbalance_generator(self.num_clients, len(data), self.imbalance)
         d_idxs = np.random.permutation(len(data))
         local_datas = np.split(d_idxs, np.cumsum(samples_per_client))[:-1]
@@ -305,22 +319,22 @@ class IDPartitioner(BasicPartitioner):
 
     Args:
         num_clients (int, optional): the number of clients
-        priority (str, optional): The value should be in set ('random', 'max', 'min'). If the number of clients is smaller than the total number of all the clients, this term will decide the selected clients according to their local data sizes.
+        priority (str, optional): The value should be in set ('random', 'max', 'min'). If the number of clients is smaller than the total number of all the clients, this term will decide the selected clients according to their local_movielens_recommendation data sizes.
     """
-    def __init__(self, num_clients=-1, priority='random'):
+    def __init__(self, num_clients=-1, priority='random', index_func=lambda X:X.id):
         self.num_clients = int(num_clients)
         self.priorty = priority
-        return
+        self.index_func = index_func
 
     def __str__(self):
         return 'id'
 
     def __call__(self, data):
         all_data = list(range(len(data)))
-        data_owners = data.id
+        data_owners = self.index_func(data)
         local_datas = collections.defaultdict(list)
         for idx in range(len(all_data)):
-            local_datas[data_owners[idx]].append(all_data[idx])
+            local_datas[data_owners[idx]].append(idx)
         local_datas = list(local_datas.values())
         if self.num_clients < 0:
             self.num_clients = len(local_datas)
@@ -331,6 +345,7 @@ class IDPartitioner(BasicPartitioner):
         elif self.priorty == 'random':
             random.shuffle(local_datas)
             local_datas = local_datas[:self.num_clients]
+        local_datas = sorted(local_datas, key=lambda x:data[x[0]][self.index_func] if self.index_func is not None else data.id[x[0]])
         return local_datas
 
 class VerticalSplittedPartitioner(BasicPartitioner):
@@ -389,4 +404,106 @@ class VerticalSplittedPartitioner(BasicPartitioner):
             return None
         for i in range(l, n + 1):
             for result in self.integer_k_partition(n - i, k - 1, i):
-                yield (i,) + result
\ No newline at end of file
+                yield (i,) + result
+
+class NodeLouvainPartitioner(BasicPartitioner):
+    """
+    Partition a graph into several subgraph by louvain algorithms. The input
+    of this partitioner should be of type networkx.Graph
+    """
+    def __init__(self, num_clients=100):
+        self.num_clients = num_clients
+
+    def __str__(self):
+        name = "Louvain"
+        return name
+
+    def __call__(self, data):
+        r"""
+        Partition graph data by Louvain algorithm and similar nodes (i.e. being of the same community) will be
+        allocated one client.
+        Args:
+            data (networkx.Graph):
+        Returns:
+            local_nodes (List): the local nodes id owned by each client (e.g. [[1,2], [3,4]])
+        """
+        local_nodes = [[] for _ in range(self.num_clients)]
+        self.node_groups = community.community_louvain.best_partition(data)
+        groups = collections.defaultdict(list)
+        for ni, gi in self.node_groups.items():
+            groups[gi].append(ni)
+        groups = {k: groups[k] for k in list(range(len(groups)))}
+        # ensure the number of groups is larger than the number of clients
+        while len(groups) < self.num_clients:
+            # find the group with the largest size
+            groups_lens = [groups[k] for k in range(len(groups))]
+            max_gi = np.argmax(groups_lens)
+            # set the size of the new group
+            min_glen = min(groups_lens)
+            max_glen = max(groups_lens)
+            if max_glen < 2 * min_glen: min_glen = max_glen // 2
+            # split the group with the largest size into two groups
+            nodes_in_gi = groups[max_gi]
+            new_group_id = len(groups)
+            groups[new_group_id] = nodes_in_gi[:min_glen]
+            groups[max_gi] = nodes_in_gi[min_glen:]
+        # allocate different groups to clients
+        groups_lens = [len(groups[k]) for k in range(len(groups))]
+        group_ids = np.argsort(groups_lens)
+        for gi in group_ids:
+            cid = np.argmin([len(li) for li in local_nodes])
+            local_nodes[cid].extend(groups[gi])
+        return local_nodes
+
+class BasicHierPartitioner(BasicPartitioner):
+    def __init__(self, Partitioner1=IIDPartitioner, pargs1:dict ={'num_clients':5}, Partitioner2=IIDPartitioner, pargs2:dict = {'num_clients':20}):
+        self.p1 = Partitioner1(**pargs1)
+        self.p2 = Partitioner2(**pargs2)
+
+    def __call__(self, data):
+        edge_servers_data = self.p1(data)
+        res = []
+        for edge_data_idx in edge_servers_data:
+            edge_data = [data[did] for did in edge_data_idx]
+            edge_local_datas = self.p2(edge_data)
+            for cid in range(len(edge_local_datas)):
+                for k in range(len(edge_local_datas[cid])):
+                    edge_local_datas[cid][k] = edge_data_idx[edge_local_datas[cid][k]]
+            res.append(edge_local_datas)
+        return res
+
+#
+# class KMeansPartitioner(BasicPartitioner):
+#     """`Partition the indices of samples in the original dataset
+#         according to their feature similarity.
+#
+#         Args:
+#             num_clients (int, optional): the number of clients
+#             imbalance (float, optional): the degree of imbalance of the amounts of different local_movielens_recommendation data (0<=imbalance<=1)
+#         """
+#
+#     def __init__(self, num_clients=100, index_func=lambda x:x[0]):
+#         self.num_clients = num_clients
+#         self.index_func = index_func
+#
+#     def __str__(self):
+#         name = ""
+#         return name
+#
+#     def __call__(self, data):
+#         features = [self.index_func(d) for d in data]
+#         if len(features)==0: raise NotImplementedError("Feature indexed by {} was not found.".format(self.index_func))
+#         if torch.is_tensor(features[0]):
+#             features = np.array([t.view(-1).cpu().numpy() for t in features])
+#         else:
+#             features = np.array(features)
+#         if len(features[0].shape)>1:
+#             features = np.array([fe.flatten() for fe in features])
+#         from scipy.cluster.vq import kmeans
+#         centers = kmeans(features, self.num_clients)
+#         local_datas = [[] for _ in self.num_clients]
+#         for did, fe in enumerate(features):
+#             cid = int((centers@fe).argmax())
+#             local_datas[cid].append(did)
+#         # local_datas = [di.tolist() for di in local_datas]
+#         return local_datas
\ No newline at end of file
